@@ -48,7 +48,7 @@ class Method(object):
     def __init__(self, name: str, func: Callable[..., JSONType],
                  mtype: MethodType = MethodType.RPCMETHOD,
                  category: str = None, desc: str = None,
-                 long_desc: str = None, deprecated: bool = False):
+                 long_desc: str = None, deprecated: Union[bool, List[str]] = None):
         self.name = name
         self.func = func
         self.mtype = mtype
@@ -217,13 +217,16 @@ class Plugin(object):
                  dynamic: bool = True,
                  init_features: Optional[Union[int, str, bytes]] = None,
                  node_features: Optional[Union[int, str, bytes]] = None,
-                 invoice_features: Optional[Union[int, str, bytes]] = None):
+                 invoice_features: Optional[Union[int, str, bytes]] = None,
+                 custom_msgs: Optional[List[int]] = None):
         self.methods = {
-            'init': Method('init', self._init, MethodType.RPCMETHOD)
+            'init': Method('init', self._init, MethodType.RPCMETHOD),
+            'setconfig': Method('setconfig', self._set_config, MethodType.RPCMETHOD)
         }
 
         self.options: Dict[str, Dict[str, Any]] = {}
         self.notification_topics: List[str] = []
+        self.custom_msgs = custom_msgs
 
         def convert_featurebits(
                 bits: Optional[Union[int, str, bytes]]) -> Optional[str]:
@@ -294,7 +297,7 @@ class Plugin(object):
                    category: Optional[str] = None,
                    desc: Optional[str] = None,
                    long_desc: Optional[str] = None,
-                   deprecated: bool = False) -> None:
+                   deprecated: Union[bool, List[str]] = None) -> None:
         """Add a plugin method to the dispatch table.
 
         The function will be expected at call time (see `_dispatch`)
@@ -324,8 +327,9 @@ class Plugin(object):
         The `category` argument can be used to specify the category of the
         newly created rpc command.
 
-        `deprecated` means that it won't appear unless `allow-deprecated-apis`
-        is true (the default).
+        `deprecated` True means that it won't appear unless `allow-deprecated-apis`
+        is true (the default), or if list of version string (e.g. "v23.08"), then
+        start deprecation cycle at that version (and removal after second entry in list).
         """
         if name in self.methods:
             raise ValueError(
@@ -386,8 +390,9 @@ class Plugin(object):
 
     def add_option(self, name: str, default: Optional[str],
                    description: Optional[str],
-                   opt_type: str = "string", deprecated: bool = False,
-                   multi: bool = False) -> None:
+                   opt_type: str = "string", deprecated: Union[bool, List[str]] = None,
+                   multi: bool = False,
+                   dynamic=False) -> None:
         """Add an option that we'd like to register with lightningd.
 
         Needs to be called before `Plugin.run`, otherwise we might not
@@ -412,10 +417,12 @@ class Plugin(object):
             'value': None,
             'multi': multi,
             'deprecated': deprecated,
+            "dynamic": dynamic
         }
 
     def add_flag_option(self, name: str, description: str,
-                        deprecated: bool = False) -> None:
+                        deprecated: Union[bool, List[str]] = None,
+                        dynamic: bool = False) -> None:
         """Add a flag option that we'd like to register with lightningd.
 
         Needs to be called before `Plugin.run`, otherwise we might not
@@ -423,7 +430,7 @@ class Plugin(object):
 
         """
         self.add_option(name, None, description, opt_type="flag",
-                        deprecated=deprecated)
+                        deprecated=deprecated, dynamic=dynamic)
 
     def add_notification_topic(self, topic: str):
         """Announce that the plugin will emit notifications for the topic.
@@ -442,7 +449,7 @@ class Plugin(object):
     def async_method(self, method_name: str, category: Optional[str] = None,
                      desc: Optional[str] = None,
                      long_desc: Optional[str] = None,
-                     deprecated: bool = False) -> NoneDecoratorType:
+                     deprecated: Union[bool, List[str]] = None) -> NoneDecoratorType:
         """Decorator to add an async plugin method to the dispatch table.
 
         Internally uses add_method.
@@ -457,7 +464,7 @@ class Plugin(object):
     def method(self, method_name: str, category: Optional[str] = None,
                desc: Optional[str] = None,
                long_desc: Optional[str] = None,
-               deprecated: bool = False) -> JsonDecoratorType:
+               deprecated: Union[bool, List[str]] = None) -> JsonDecoratorType:
         """Decorator to add a plugin method to the dispatch table.
 
         Internally uses add_method.
@@ -659,10 +666,13 @@ class Plugin(object):
             self.log(traceback.format_exc())
 
     def _dispatch_notification(self, request: Request) -> None:
-        if request.method not in self.subscriptions:
-            raise ValueError("No subscription for {name} found.".format(
-                name=request.method))
-        func = self.subscriptions[request.method]
+        if request.method in self.subscriptions:
+            func = self.subscriptions[request.method]
+        # Wildcard 'all' subscriptions using asterisk
+        elif '*' in self.subscriptions:
+            func = self.subscriptions['*']
+        else:
+            raise ValueError(f"No subscription for {request.method} found.")
 
         try:
             self._exec_func(func, request)
@@ -674,7 +684,6 @@ class Plugin(object):
         # then utf8 ourselves.
         s = bytes(json.dumps(
             obj,
-            cls=LightningRpc.LightningJSONEncoder,
             ensure_ascii=False
         ) + "\n\n", encoding='utf-8')
         with self.write_lock:
@@ -779,7 +788,7 @@ class Plugin(object):
         """)
 
         for method in self.methods.values():
-            if method.name in ['init', 'getmanifest']:
+            if method.name in ['init', 'getmanifest', 'setconfig']:
                 # Skip internal methods provided by all plugins
                 continue
 
@@ -859,7 +868,7 @@ class Plugin(object):
         hooks = []
         for method in self.methods.values():
             # Skip the builtin ones, they don't get reported
-            if method.name in ['getmanifest', 'init']:
+            if method.name in ['getmanifest', 'init', 'setconfig']:
                 continue
 
             if method.mtype == MethodType.HOOK:
@@ -912,7 +921,7 @@ class Plugin(object):
                 m["long_description"] = method.long_desc
 
         manifest = {
-            'options': list(self.options.values()),
+            'options': list({k: v for k, v in d.items() if v is not None} for d in self.options.values()),
             'rpcmethods': methods,
             'subscriptions': list(self.subscriptions.keys()),
             'hooks': hooks,
@@ -928,6 +937,9 @@ class Plugin(object):
         if features is not None:
             manifest['featurebits'] = features
 
+        if self.custom_msgs is not None:
+            manifest['custommessages'] = self.custom_msgs
+
         return manifest
 
     def _init(self, options: Dict[str, JSONType],
@@ -937,15 +949,15 @@ class Plugin(object):
         def verify_str(d: Dict[str, JSONType], key: str) -> str:
             v = d.get(key)
             if not isinstance(v, str):
-                raise ValueError("Wrong argument to init: expected {key} to be"
-                                 " a string, got {v}".format(key=key, v=v))
+                raise TypeError("Wrong argument to init: expected {key} to be"
+                                " a string, got {v}".format(key=key, v=v))
             return v
 
         def verify_bool(d: Dict[str, JSONType], key: str) -> bool:
             v = d.get(key)
             if not isinstance(v, bool):
-                raise ValueError("Wrong argument to init: expected {key} to be"
-                                 " a bool, got {v}".format(key=key, v=v))
+                raise TypeError("Wrong argument to init: expected {key} to be"
+                                " a bool, got {v}".format(key=key, v=v))
             return v
 
         self.rpc_filename = verify_str(configuration, 'rpc-file')
@@ -962,6 +974,12 @@ class Plugin(object):
             return self._exec_func(self.child_init, request)
         return None
 
+    def _set_config(self, **_) -> None:
+        """Called when the value of a dynamic option is changed
+        For now we don't do anything.
+        """
+        pass
+
 
 class PluginStream(object):
     """Sink that turns everything that is written to it into a notification.
